## Forecast code for WGCSE2021 - Updated for WGCSE 2020 following IBPcSWhiting 2021
## DS 23/10/2020


library(stockassessment)
load("run/model.RData")
source("src/fixfore.R")

nosim <-50001 #number of simulations (i think 1001 is defaut, 10001 better, 101 for quick check)
# Note - the number of iterations 10001, up from 1001. All are stable if seed is set - standard "12345" used.
# set to 101 for quick checking
# set to 1001 for routine checking
# set to 50001 preference
deterministic <- F # Leave deterministic = F to default to stochastic forecast


###############################
## Reference points 

Fmsy <- 0.375			# Median point estimate of (F05) EqSim with segreg S/R model
Fmsyupper <- 0.375 	## ICES Advice rule (capped @ Fmsy)
Fmsylower <- 0.315 	## ICES Advice rule

Flim <- 0.64 		# Can't be calculated so using Fmsy instead	## Chk
Fpa <- 0.375		# Can't estimated so ICES uses Fmsy	## Chk
Bpa <- 50818		# Btrigger
Blim <- 36571		# Bpa minus erro
Btrigger <- 50818	# Bpa

F05 <- 0.375			## Without Btrigger rule
Fpa <- min(F05, Fpa)
Fpa_x <- 0.375

## Get F status quo
Fstatquo <- tail(fbartable(fit),0)

## Fsq for last 3 yrs 
## FLR version: F_SQ <- mean(fbar(stock)[,as.character(stock@range['maxyear']-2:0)]) # last 3 years - normal default
Flast <- tail(fbartable(fit),1)[,"Estimate"]     # F in the last year of the assessment
Fsq <- mean(tail(fbartable(fit),3)[,"Estimate"]) # Assumed F in the intermediate year
#Fsq <- Flast # scale to the last year


###############################
## Forecast Assumptions

lastyear = max(fit$data$years)	# Final year in assessment, first year forecast (intermediate year)
## Recruitment years (vector covering full survey series) 
Ry <- 2010:2019			# Resample R from previous years (random walk), start of the time series to end of time series (median is calculated)

## biological vars years
# use last 3 years for fishing selectivity and for average weight, maturity, M
Ay <- lastyear +(-2:0) # this is last three years
###############################



###############################
## TAC & advice for 2020 
prev.TAC <- 19184 	# 2019 TAC
TAC <- 10863 		# 2020 TAC




#############################################################################################################################################################################
##  **MixedFish Catch constraint for 2020 for intermediate year**
#############################################################################################################################################################################

# Whiting catch when Haddock @ FMSY (updated by CM in plenary WGCSE21 6/5/21)
#MixFish <-8678

## !! Due to being a revision we are using the actual 2020 catch instead of prediction from MixFish 
MixFish <-7197
Fmix <- 0.385

###############################################################################################################################################################################








###############################
## forecast names and structure part 1
## Forecasts
# main scenarios (values are for 2019:2022)

FC<-list()


          set.seed(12345)
          FC[[length(FC)+1]] <- forecast.fix(fit, label = "MSY approach: Fmsy", fval = c(Flast,NA,Fmsy,Fmsy), catchval.exact = c(NA, MixFish, NA, NA), ave.years=Ay, rec.years=Ry, splitLD=TRUE, nosim = nosim, deterministic=deterministic)

          set.seed(12345)
          FC[[length(FC)+1]] <- forecast.fix(fit, label = "Precautionary approach: Fpa", fval = c(Flast,NA,Fpa,Fpa), catchval.exact = c(NA, MixFish, NA, NA), ave.years=Ay, rec.years=Ry, splitLD=TRUE, nosim = nosim,deterministic=deterministic)
          set.seed(12345)
          FC[[length(FC)+1]] <- forecast.fix(fit, label = "FMSY upper", fval = c(Flast,NA,Fmsyupper,Fmsyupper), catchval.exact = c(NA, MixFish, NA, NA), ave.years=Ay, rec.years=Ry, splitLD=TRUE, nosim = nosim,deterministic=deterministic)
          set.seed(12345)
          FC[[length(FC)+1]] <- forecast.fix(fit, label = "FMSY lower", fval = c(Flast,NA,Fmsylower,Fmsylower), catchval.exact = c(NA, MixFish, NA, NA), ave.years=Ay, rec.years=Ry, splitLD=TRUE, nosim = nosim,deterministic=deterministic)
          set.seed(12345)
          FC[[length(FC)+1]] <- forecast.fix(fit, label = "F = 0", fval = c(Flast,NA,0.000001, 0.000001), catchval.exact = c(NA, MixFish, NA, NA), ave.years=Ay, rec.years=Ry, splitLD=TRUE, nosim = nosim,deterministic=deterministic)
          set.seed(12345)
          FC[[length(FC)+1]] <- forecast.fix(fit, label = "Fpa", fval = c(Flast,NA,Fpa_x,Fpa_x), catchval.exact = c(NA, MixFish, NA, NA), ave.years=Ay, rec.years=Ry, splitLD=TRUE, nosim = nosim,deterministic=deterministic)
          
          #set.seed(12345)
          #FC[[length(FC)+1]] <- forecast(fit, label = "F = Flim", fval = c(Flast,Fmix,Flim,Flim), ave.years=Ay, rec.years=Ry, splitLD=TRUE, nosim = nosim,deterministic=deterministic)
          
          set.seed(12345)
          FC[[length(FC)+1]] <- forecast.fix(fit, label = "Fsq", fval = c(Flast,NA,NA,NA), catchval.exact = c(NA, MixFish, MixFish, MixFish), ave.years=Ay, rec.years=Ry, splitLD=TRUE, nosim = nosim,deterministic=deterministic)
          set.seed(12345)
          FC[[length(FC)+1]] <- forecast.fix(fit, label = "hit Blim", fval = c(Flast,NA, NA, NA),  catchval.exact = c(NA, MixFish, NA, NA), nextssb = c(NA, NA, Blim, Blim), ave.years=Ay, rec.years=Ry, splitLD=TRUE, nosim = 						nosim,deterministic=deterministic)
          
          #set.seed(12345)
          #FC[[length(FC)+1]] <- forecast(fit, label =  "hit Bpa", fval = c(Flast,Fmix, NA, NA), nextssb = c(NA, NA, Bpa, Bpa), ave.years=Ay, rec.years=Ry, splitLD=TRUE, nosim = nosim,deterministic=deterministic)
          #set.seed(12345)
          #FC[[length(FC)+1]] <- forecast(fit, label =  "hit MSY Btrigger", fval = c(Flast,Fmix, NA, NA), nextssb = c(NA, NA, Btrigger, Btrigger), ave.years=Ay, rec.years=Ry, splitLD=TRUE, nosim = nosim,deterministic=deterministic)
          #set.seed(12345)
          #FC[[length(FC)+1]] <- forecast.fix(fit, label =  "Take TAC in 2020, then Fmsy", fval = c(Flast,NA, Fmsy, Fmsy), catchval = c(NA,TAC,NA,NA), ave.years=Ay, rec.years=Ry, splitLD=TRUE, nosim = nosim,deterministic=deterministic)
          set.seed(12345)
          FC[[length(FC)+1]] <- forecast.fix(fit, label =  "F=F2020", fval = c(Flast, NA, Fmix, Fmix), catchval.exact = c(NA, MixFish, NA, NA),  ave.years=Ay, rec.years=Ry, splitLD=TRUE, nosim = nosim,deterministic=deterministic)
          

	 
### Where SSB < MSY Btrigger (50818 t)
###############################################################################################
# SSB in Advice Year (2021)
SSBadv <- 33487
Fadj <- Fmsy*(SSBadv/Btrigger)
Flower <- Fmsylower*(SSBadv/Btrigger)
Fupper <- Fmsyupper*(SSBadv/Btrigger)
###############################################################################################


      set.seed(12345)
      FC[[length(FC)+1]] <- forecast.fix(fit, label = "ICES Advice Basis", fval = c(Flast,NA,Fadj,Fmsy), catchval.exact = c(NA, MixFish, NA, NA), ave.years=Ay, rec.years=Ry, splitLD=TRUE, nosim = nosim,deterministic=deterministic)
      set.seed(12345)
      FC[[length(FC)+1]] <- forecast.fix(fit, label = "ICES Advice lower", fval = c(Flast,NA,Flower,Fmsy), catchval.exact = c(NA, MixFish, NA, NA), ave.years=Ay, rec.years=Ry, splitLD=TRUE, nosim = nosim,deterministic=deterministic)
      set.seed(12345)
      FC[[length(FC)+1]] <- forecast.fix(fit, label = "ICES Advice upper", fval = c(Flast,NA,Fupper,Fmsy), catchval.exact = c(NA, MixFish, NA, NA), ave.years=Ay, rec.years=Ry, splitLD=TRUE, nosim = nosim,deterministic=deterministic)




for(i in 1:length(FC)){
    tmp = FC[[i]]
    maxy =  max( unlist(lapply(tmp,function(xx) xx$year)))
    FC[[i]] = list(  list(year=maxy) )
    attributes(FC[[i]]) <- attributes(tmp)
}
save(FC, file="run/forecast.RData")
